// this library is public domain. enjoy!
// www.ladyada.net/learn/sensors/thermocouple

#include <TexasInstruments\lm4f120h5qr.h>
#include "MAX6675.h"
#include  "Ctypes.h"

#define MOD_TC_to_UEXT1

#ifdef MOD_TC_to_UEXT1
  //Definitions for UEXT1 connection:
  #define UEXT_PWR_En     GPIO_PORTE_DATA_R &= (~0x0004);
  #define UEXT_PWR_Dis    GPIO_PORTE_DATA_R |= 0x0004; CS_Low; SCK_Low;
  #define CS_High         GPIO_PORTD_DATA_R |= 0x0002;
  #define CS_Low          GPIO_PORTD_DATA_R &= (~0x0002);
  #define SCK_High        GPIO_PORTD_DATA_R |= 0x0001;
  #define SCK_Low         GPIO_PORTD_DATA_R &= (~0x0001);
  #define MISO_Read       (GPIO_PORTD_DATA_R & 0x0004)

#else
  //Definitions for UEXT2 connection:
  #define UEXT_PWR_En     GPIO_PORTE_DATA_R &= (~0x0008);
  #define UEXT_PWR_Dis    GPIO_PORTE_DATA_R |= 0x0008; CS_Low; SCK_Low;
  #define CS_High         GPIO_PORTB_DATA_R |= 0x0020;
  #define CS_Low          GPIO_PORTB_DATA_R &= (~0x0020);
  #define SCK_High        GPIO_PORTB_DATA_R |= 0x0010;
  #define SCK_Low         GPIO_PORTB_DATA_R &= (~0x0010);
  #define MISO_Read       (GPIO_PORTB_DATA_R & 0x0060)

#endif

#define NAN             'N'
#define Delay_Time      1000


static void delay(volatile long cycles)
{
  while(cycles){cycles--;}
}


void Init_MAX6675(void)
{ 
  // Initialization
#ifdef MOD_TC_to_UEXT1
  SYSCTL_RCGCGPIO_R |= (SYSCTL_RCGCGPIO_R3);  // PortD Clock Enabled: GPIO Port D Run Mode Clock
  SYSCTL_RCGCGPIO_R |= (SYSCTL_RCGCGPIO_R4);  // PortE Clock Enabled: GPIO Port E Run Mode Clock
  GPIO_PORTE_DIR_R |= 0x0004;                 // PORTE<2> UEXT1 power enable is outputs
  GPIO_PORTD_DIR_R |= 0x0003;                 // PORTD<0..1> are Outputs -> CS and SCK are outputs
  GPIO_PORTD_DIR_R &= ~(0x0004);              // PORTD<2> MISO to be input
  GPIO_PORTD_DEN_R |= 0x0007;                  // GPIO Digitals are Enabled
  GPIO_PORTE_DEN_R |= 0x0004;                  // GPIO Digital is Enabled
#else  
  SYSCTL_RCGCGPIO_R |= (SYSCTL_RCGCGPIO_R1);  // PortB Clock Enabled: GPIO Port B Run Mode Clock
  SYSCTL_RCGCGPIO_R |= (SYSCTL_RCGCGPIO_R4);  // PortE Clock Enabled: GPIO Port E Run Mode Clock
  GPIO_PORTE_DIR_R |= 0x0008;                 // PORTE<3> UEXT2 power enable is outputs
  GPIO_PORTB_DIR_R |= 0x0030;                 // PORTB<4..5> are Outputs -> CS and SCK are outputs
  GPIO_PORTB_DIR_R &= ~(0x0040);              // PORTB<6> MISO to be input
  GPIO_PORTB_DEN_R |= 0x0070;                 // GPIO Digitals are Enabled
  GPIO_PORTE_DEN_R |= 0x0008;                 // GPIO Digital is Enabled
#endif
  
  UEXT_PWR_En;
  CS_High;
  
  //define pin modes
  //pinMode(cs, OUTPUT);
  //pinMode(sclk, OUTPUT); 
  //pinMode(miso, INPUT);
  //digitalWrite(cs, HIGH);
}


long MAX6675_readCelsius(void)
{

  uint16_t v;

  CS_Low;
  delay(Delay_Time); //Wait 1ms!!!
  

  v = spiread();
  v <<= 8;
  v |= spiread();

  CS_High;

  if (v & 0x4) {
    // uh oh, no thermocouple attached!
    return NAN; 
    //return -100;
  }

  v >>= 3;

  return (long)(v*0.25);
}


long MAX6675_readFarenheit(void)
{
  return MAX6675_readCelsius() * 9/5 + 32;
}


byte spiread(void)
{ 
  int i;
  byte d = 0;

  for (i=7; i>=0; i--)
  {
    SCK_Low;
    delay(Delay_Time);  //Wait 1ms!!!
    if (MISO_Read) {
      //set the bit to 1 no matter what
      d |= (1 << i);
    }
    SCK_High;
    delay(Delay_Time);  //Wait 1ms!!!
  }

  return d;
}
